# mongodb-querybuilder

MongoDB query builder for aggregations

## Install

```
npm install
```

## Test

```
npm test
```

And then open [http://localhost:8080/__zuul](http://localhost:8080/__zuul) in a browser.

## Dist

Of course the preferred way to use -querybuilder is npm+browserify, but sometimes
being able to just drop a script tag into a codepen is nedded.

1. Create a [GitHub Access Token](https://help.github.com/articles/creating-an-access-token-for-command-line-use)
1. Export your access token as `GITHUB_TOKEN={{YOUR TOKEN}}`
1. `npm run-script dist`

## Todo

- [ ] Setup travis for saucelabs.  see [zuul docs](https://github.com/defunctzombie/zuul/wiki/travis-ci)
- [ ] Write lots more tests

## Example

The query builder turns that:

```js
builder
    .match("fields.reporter.name", ["thomasr", "ramon.fernandez", "spencer"])
    .match("fields.components.name", ["Security", "Sharding"])
    .match("changelog.total", [10, 50])
    .group("x-axis", ["fields.fixVersions.name", "fields.status"])
        .agg("y-axis", "$sum", 1)
        .agg("size", "$avg", "changelog.total")
        .agg("_ids", "$push", "_id")
    .limit(5);
```

into that:

```json
[
    {
        "$match": {
            "fields.reporter.name": {
                "$in": [
                    "thomasr",
                    "ramon.fernandez",
                    "spencer"
                ]
            },
            "fields.components.name": {
                "$in": [
                    "Security",
                    "Sharding"
                ]
            },
            "changelog.total": {
                "$gte": 10,
                "$lte": 50
            }
        }
    },
    {
        "$unwind": "$fields.fixVersions"
    },
    {
        "$project": {
            "fields_fixVersions_name": "$fields.fixVersions.name",
            "fields_status": "$fields.status",
            "changelog_total": "$changelog.total",
            "_id": "$_id"
        }
    },
    {
        "$group": {
            "_id": {
                "fields_fixVersions_name": "$fields_fixVersions_name",
                "fields_status": "$fields_status"
            },
            "y-axis": {
                "$sum": 1
            },
            "size": {
                "$avg": "$changelog_total"
            },
            "_ids": {
                "$push": "$_id"
            }
        }
    },
    {
        "$limit": 5
    },
    {
        "$project": {
            "y-axis": "$y-axis",
            "size": "$size",
            "_ids": "$_ids",
            "x-axis": "$_id",
            "_id": 0
        }
    }
]
```


Full example (see also [index.html](./index.html)): 

```html
<!DOCTYPE html>
<html>
<head>
    <title>QueryBuilder Test</title>
</head>
<body>

<script type="text/javascript" src="./dist/mongodb-querybuilder.js"></script>

<script>
// create query builder and point it to a mongodb instance and namespace
var builder = new QueryBuilder({seed: "mongodb://localhost:10000", namespace: "xgen.jira"});

// call .match, .group, .agg and .limit functions as much as you want
// QueryBuilder will store the state according to the given "slot" (first), overwrite
// old values, and delete the slot if you pass in a value of null.

builder
    .match("fields.reporter.name", ["thomasr", "ramon.fernandez", "spencer"])
    .match("fields.components.name", ["Security", "Sharding"]) 
    .match("changelog.total", [0, 50])  // for numbers, specify min and max as array

    // --- either use .group() / .agg()

    .group("x-axis", ["fields.fixVersions.name", "fields.status"])
        .agg("y-axis", "$sum", 1)
        .agg("size", "$avg", "changelog.total")
        .agg("_ids", "$push", "_id")

    // --- or .pick(), but not both.

    // .pick("x-axis", "fields.fixVersions.name")
    // .pick("y-axis", "changelog.total")
    // .pick("color", "fields.components.name")

    .sort("size", -1)
    .limit(5);

// finally, call .end to send the aggregation pipeline and return the data
builder.end(function (err, res) {
    if (err) return console.log("ERROR", err);
    console.log("DATA", JSON.stringify(res, null, '\t'));
});
</script>

</body>
</html>



