var assert = require('assert'),
  QueryBuilder = require('../');

describe('QueryBuilder', function(){
  var builder;

  before(function(done){
    // create query builder and point it to a mongodb instance and namespace
    builder = new QueryBuilder({seed: 'mongodb://localhost:10000', namespace: 'xgen.jira'});
    done();
  });

  it('should work without specifying a sort', function(done){
    builder
      .match('changelog.total', [0, 50])
      .group('x-axis', ['fields.fixVersions.name', 'fields.status'])
          .agg('y-axis', '$sum', 1)
      
      .end(function(err, res){
        assert.ifError(err);
        console.log('DATA', JSON.stringify(res, null, '\t'));
        done();
      });
  });


  it('should allow picking fields without group/agg', function(done){
    builder
      .reset()
      .match('changelog.total', [0, 50])
      .pick("x-axis", "fields.fixVersions.name")
      .pick("y-axis", "changelog.total")
      .pick("color", "fields.components.name")
      
      .end(function(err, res){
        assert.ifError(err);
        console.log('DATA', JSON.stringify(res, null, '\t'));
        done();
      });
  });

  it('should accept two group functions with different slots', function(done){
    builder
      .reset()
      .match('changelog.total', [0, 50])
      .group("x-axis", "fields.status")
          .agg("y-axis", "$sum", 1)
          .agg("size", "$avg", "changelog.total")
          .agg("_ids", "$push", "_id")
      .group("stacks", "fields.components.name")

      .end(function(err, res){
        assert.ifError(err);
        console.log('DATA', JSON.stringify(res, null, '\t'));
        done();
      });
  });


});
